pub struct MessageOverlapMerge { /* private fields */ }Expand description
Detects network partitions by comparing message hashes across DHT records.
Joins peers whose published hashes do not overlap with local message history.
Implementations
Sourceimpl MessageOverlapMerge
impl MessageOverlapMerge
Sourcepub fn new(
record_publisher: RecordPublisher,
gossip_sender: GossipSender,
gossip_receiver: GossipReceiver,
cancel_token: CancellationToken,
max_join_peers: usize,
initial_interval: Duration,
base_interval: Duration,
max_jitter: Duration,
) -> Result<Self>
pub fn new( record_publisher: RecordPublisher, gossip_sender: GossipSender, gossip_receiver: GossipReceiver, cancel_token: CancellationToken, max_join_peers: usize, initial_interval: Duration, base_interval: Duration, max_jitter: Duration, ) -> Result<Self>
Create a new split-brain detector.